@@ -134,11 +134,12 @@ class Subscription(object):
 class PreseemGrpcClient(object):
     """Implement the GRPC client to the Preseem API, adapted to asyncio."""
 
-    def __init__(self, api_key, host=None, port=None):
+    def __init__(self, api_key, host=None, port=None, compression=False):
         self.api_key = api_key
         self._executor = concurrent.futures.ThreadPoolExecutor(max_workers=10)
         self.host = host or 'nmapi.preseem.com'
         self.port = port or 443
+        self.compression = compression
         self._channel = None
         self._metrics_svc = None
         self._nm_svc = None
@@ -282,6 +283,9 @@ class PreseemGrpcClient(object):
                      True),  # send keepalives when no grpc calls
                     ('grpc.http2.max_pings_without_data', 0)
                 ]  # allow unlimited pings without data
+                if self.compression:
+                    grpc_opts.append(
+                        ('grpc.default_compression_algorithm', grpc.Compression.Gzip))
                 if self.api_key:
                     call_creds = grpc.metadata_call_credentials(
                         ApiKeyCallCredentials(self.api_key))
