@@ -118,7 +118,7 @@ class ConfigModel(object):
             if f == 'extra':
                 return extra
             val = getattr(pb, f, None)
-            if field_is_list.get(f):
+            if field_is_list.get(f) and (config_type, f) not in self._map_types:
                 val = list(val)
             elif f in nested_fields:
                 if (config_type, f) in self._map_types:
