@@ -1,9 +1,10 @@
 """Data structures used to manipulate the model locally within a data source."""
 from collections import namedtuple
 from ipaddress import ip_address, ip_network
+import time
 from uuid import UUID
 
-from .grpc_nm_model import NetworkMetadataReference
+from .grpc_nm_model import NetworkMetadataEntity, NetworkMetadataReference, Reference
 from .util import normalize_mac
 from preseem_protobuf.model import service_pb2
 from preseem_protobuf.model.account_pb2 import AccountStatus
@@ -264,6 +265,63 @@ class Account(Errorable):
         return hash(self.id)
 
 
+class Site(Errorable):
+    """A Site object."""
+    typename = 'site'
+    __slots__ = ["_id", "_name", "_network_prefixes"]
+    attrs = ('id', 'name', 'network_prefixes')
+
+    def __init__(self, id, **kwargs):
+        super().__init__()
+        if id is None or str(id) == '':
+            raise ValueError("The 'id' parameter must be non-None.")
+        self._id = str(id)
+        self._name = None
+        self._network_prefixes = []
+        for key, value in kwargs.items():
+            setattr(self, key, value)
+
+    @property
+    def id(self):
+        return self._id
+
+    @property
+    def name(self):
+        return self._name
+
+    @name.setter
+    def name(self, value):
+        if self._check_str('name', value):
+            self._name = value if value is not None else None
+
+    @property
+    def network_prefixes(self):
+        return self._network_prefixes
+
+    @network_prefixes.setter
+    def network_prefixes(self, value):
+        if value is not None:
+            if not isinstance(value, list):
+                raise TypeError("The 'netwock_prefixes' parameter must be a list.")
+        self._network_prefixes = value or []
+
+    def __repr__(self):
+        attr_values = [(attr, getattr(self, attr)) for attr in self.attrs]
+        attributes_str = ', '.join(f"{attr}={value!r}" for attr, value in attr_values)
+        return f"{type(self).__name__}({attributes_str})"
+
+    def __eq__(self, other):
+        if isinstance(other, self.__class__):
+            for slot in self.__slots__:
+                if getattr(self, slot) != getattr(other, slot):
+                    return False
+            return True
+        return False
+
+    def __hash__(self):
+        return hash(self.id)
+
+
 class Attachment(Errorable):
     """A Service Attachment object that can be easily manimulated in Python."""
     typename = 'attachment'
@@ -553,7 +611,15 @@ class Service(Errorable):
 class BaseNetworkElement(Errorable):
     """Base class for NetworkElement types to use within a data source."""
     __slots__ = [
-        "_id", "_community", "_host", "_name", "_site", "_url", "_site_uuid", "_uuid"
+        "_id",
+        "_community",
+        "_host",
+        "_name",
+        "_site",
+        "_url",
+        "_site_uuid",
+        "_uuid",
+        "_site_id",
     ]
     # this is just used to list fields we want to print with __repr__
     attrs = ('id', 'name', 'site', 'host', 'community', 'url')
@@ -570,6 +636,7 @@ class BaseNetworkElement(Errorable):
         self._url = None
         self._site_uuid = None  # used by sync_preseem_model for resolution
         self._uuid = None  # used to pass uuid back from sync_preseem_model
+        self._site_id = None  # used for old topology mapping to a site
         for key, value in kwargs.items():
             setattr(self, key, value)
 
@@ -644,6 +711,15 @@ class BaseNetworkElement(Errorable):
         if self._check_str('url', value):
             self._url = value if value is not None else None
 
+    @property
+    def site_id(self):
+        return self._site_id
+
+    @site_id.setter
+    def site_id(self, value):
+        if self._check_str('site_id', value):
+            self._site_id = value if value is not None else None
+
     def __repr__(self):
         attr_values = [(attr, getattr(self, attr)) for attr in self.attrs]
         attributes_str = ', '.join(f"{attr}={value!r}" for attr, value in attr_values)
@@ -661,6 +737,49 @@ class BaseNetworkElement(Errorable):
         return hash(self.id)
 
 
+class AccessPoint(BaseNetworkElement):
+    """AP class to use within a data source."""
+    typename = 'ap'
+    role_uuid = UUID('ae3f1aa5-063a-4a24-9744-822a82d107d9')
+
+    def get_nm(self, system=None):
+        """Return network metadata objects for an Access Point."""
+        ref = NetworkMetadataReference(self.typename, self._id, {})
+        if self._host:
+            ref.attributes['ip_address'] = self._host
+            ref.attributes['ap_config'] = Reference(self._id)
+            ref.attributes['topology1_snmp'] = Reference(self._id)
+        if self._name:
+            ref.attributes['topology1'] = self._name
+        if self._site:
+            ref.attributes['topology2'] = self._site
+        if self._community:
+            ref.attributes['community'] = self._community
+        if self._uuid:
+            ref.attributes['uuid'] = str(UUID(bytes=self._uuid))
+        if system:
+            ref.attributes['system'] = system
+        objs = [ref]
+        if self._host:
+            attrs = {
+                'service_rate_down': 10000000,
+                'service_rate_up': 10000000,
+                'subscriber_identifier': self._host,
+                'subscriber_name': self._name
+            }
+            if system:
+                attrs['system'] = system
+            if self._site_id:
+                attrs['ap'] = Reference(self._site_id)
+            objs.append(NetworkMetadataReference('session', self._host, attrs))
+            objs.append(
+                NetworkMetadataEntity(self._host, time.time(), {
+                    'session': Reference(self._host),
+                    'snmp_session': Reference(self._host)
+                }))
+        return objs
+
+
 class Olt(BaseNetworkElement):
     """OLT class to use within a data source."""
     typename = 'olt'
